@@ -8,7 +8,7 @@ use warnings;
 use Carp;
 use Encode;
 
-our $VERSION = '0.022';
+our $VERSION = '0.023';
 
 require XSLoader;
 XSLoader::load('HarfBuzz::Shaper', $VERSION);
@@ -34,7 +34,7 @@ HarfBuzz::Shaper is a perl module that provides access to a small
 subset of the native HarfBuzz library.
 
 The subset is suitable for typesetting programs that need to deal with
-complex languages like Devanagari.
+complex languages like Devanagari, Hebrew or Arabic.
 
 This module is intended to be used with module L<Text::Layout>. Feel
 free to (ab)use it for other purposes.
@@ -53,6 +53,15 @@ following items:
 Note that the number of glyphs does not necessarily match the number
 of input characters!
 
+=head1 DISCLAIMER
+
+This module provides a thin interface layer between Perl and the
+native HarfBuzz library. It is agnostic with regard to the details of
+multi-language typesetting. HarfBuzz has a friendly community to help
+you.
+
+L<https://lists.freedesktop.org/mailman/listinfo/harfbuzz>
+
 =head1 METHODS
 
 =head2 $hb = HarfBuzz::Shaper->new( [ options ] )
@@ -95,11 +104,39 @@ sub new {
     return $self;
 }
 
+=head2 $hb->reset( [ I<full> ] )
+
+Reset (clear) the buffer settings for font, size, language, direction
+and script. With I<full>, also clears the font cache.
+
+=cut
+
+sub reset {
+    my ( $self, $full ) = @_;
+
+    for ( qw ( font size text language direction script ) ) {
+	delete $self->{$_};
+    }
+    if ( $full ) {
+	for ( keys %$self ) {
+	    next unless /^(font|face)_/;
+	    delete $self->{$_};
+	}
+	hb_buffer_reset( $self->{buffer} );
+	# So basically we are like freshly created.
+    }
+
+    $self;
+}
+
 =head2 $hb->set_font( I<font filename> [ , I<size> ] )
 
 Explicit way to set the font (and, optionally, the size) used for
 shaping.
 
+The settings persist across shaper() calls. Call without arguments to
+remove the settings.
+
 The font must be a TrueType or OpenType font. Font information is
 cached internally, after the first call subsequent calls with the same
 font filename are very fast.
@@ -109,6 +146,12 @@ font filename are very fast.
 sub set_font {
     my ( $self, $fontfile, $size ) = @_;
 
+    unless ( defined $fontfile or defined $size ) {
+	delete $self->{font};
+	delete $self->{size};
+	return $self;
+    }
+
     croak("$fontfile: $!\n") unless -s -r $fontfile;
     my $blob = hb_blob_create_from_file($fontfile);
     my $face = $self->{"face_$fontfile"} //= hb_face_create( $blob, 0 );
@@ -131,11 +174,19 @@ Note that the font size will in general affect details of the
 appearance, A 5 point fontsize magnified 10 times is not identical to
 50 point font size.
 
+The setting persist across shaper() calls. Call without arguments to
+remove the setting.
+
 =cut
 
 sub set_size {
     my ( $self, $size ) = @_;
 
+    unless ( defined $size ) {
+	delete $self->{size};
+	return $self;
+    }
+
     $self->{size} = $size;
 
     $self;
@@ -143,15 +194,23 @@ sub set_size {
 
 =head2 $hb->set_text( I<text> [ , ... ] )
 
-Set the text to shape. Multiple arguments are concatenated.
+Sets the text to shape. Multiple arguments are concatenated.
 
 Note that the text must be Perl strings.
 
+The setting persist across shaper() calls. Call without arguments to
+remove the setting.
+
 =cut
 
 sub set_text {
     my ( $self, @text ) = @_;
 
+    unless ( @_ > 1 and defined $text[0] ) {
+	delete $self->{text};
+	return $self;
+    }
+
     $self->{text} = join( "", @text );
 
     $self;
@@ -160,16 +219,21 @@ sub set_text {
 =head2 $hb->set_features( I<feat> [ , ... ] )
 
 Sets persistent features for shaping. Features are strings as described in
-L<https://harfbuzz.github.io/harfbuzz-hb-common.html#hb-feature-from-string>.
+L<https://harfbuzz.github.io/harfbuzz-hb-common.html#hb-feature-from-string>
+and
+L<https://css-tricks.com/almanac/properties/f/font-feature-settings/#values>.
 
 Multiple feature strings may be supplied.
 
+Call without arguments to remove the persistent features.
+
 =cut
 
 sub set_features {
     my ( $self ) = shift;
     $self->{features} = [];
-    $self->add_features(@_);
+    $self->add_features(@_) if @_ && defined($_[0]);
+    return $self;
 }
 
 =head2 $hb->add_features( I<feat> [ , ... ] )
@@ -193,10 +257,21 @@ sub add_features {
 Sets the language for shaping. I<lang> must be a string containing a
 valid BCP-47 language code.
 
+The setting persist across shaper() calls. Call without arguments to
+remove the setting.
+
 =cut
 
 sub set_language {
     my ( $self, $lang ) = @_;
+
+    unless ( defined $lang ) {
+	delete $self->{language};
+	return $self;
+    }
+
+    $self->{language} = $lang;
+    # This is merely for checking validity;
     hb_buffer_set_language( $self->{buffer}, $lang );
 }
 
@@ -204,6 +279,9 @@ sub set_language {
 
 Returns the language currently set for this shaper, as a string.
 
+When called after a successful shaper() call, it returns the actual
+value used by shaper().
+
 =cut
 
 sub get_language {
@@ -211,9 +289,91 @@ sub get_language {
     hb_buffer_get_language( $self->{buffer} );
 }
 
+=head2 $hb->set_script( I<script> )
+
+Sets the script (alphabet) for shaping. I<script> must be a string
+containing a valid ISO-15924 script code. For example, C<"Latn"> for
+the Latin (Western European) script, or C<"Arab"> for arabic script.
+
+If you don't set a script, shaper() will make a guess based on the
+text string. This may or may not yield desired results.
+
+The setting persist across shaper() calls. Call without arguments to
+remove the setting.
+
+=cut
+
+sub set_script {
+    my ( $self, $script ) = @_;
+
+    unless ( defined $script ) {
+	delete $self->{script};
+	return $self;
+    }
+
+    $self->{script} = $script;
+    # This is merely for checking validity;
+    hb_buffer_set_script( $self->{buffer}, $script );
+}
+
+=head2 $hb->get_script
+
+Returns the script currently set for this shaper, as a string.
+
+When called after a successful shaper() call, it returns the actual
+value used by shaper().
+
+=cut
+
+sub get_script {
+    my ( $self ) = @_;
+    hb_buffer_get_script( $self->{buffer} );
+}
+
+=head2 $hb->set_direction( I<dir> )
+
+Sets the direction for shaping. I<dir> must be a string containing a
+valid direction setting: LTR (left-to-right), RTL (right-to-left), TTB
+(top-to-bottom), or BTT (bottom-to-top).
+
+If you don't set a direction, shaper() will make a guess based on the
+text string. This may or may not yield desired results.
+
+The setting persist across shaper() calls. Call without arguments to
+remove the setting.
+
+=cut
+
+sub set_direction {
+    my ( $self, $dir ) = @_;
+
+    unless ( defined $dir ) {
+	delete $self->{direction};
+	return $self;
+    }
+
+    $self->{direction} = $dir;
+    # This is merely for checking validity;
+    hb_buffer_set_direction( $self->{buffer}, $dir );
+}
+
+=head2 $hb->get_direction
+
+Returns the direction currently set for this shaper, as a string.
+
+When called after a successful shaper() call, it returns the actual
+value used by shaper().
+
+=cut
+
+sub get_direction {
+    my ( $self ) = @_;
+    hb_buffer_get_direction( $self->{buffer} );
+}
+
 =head2 $info = $hb->shaper( [ I<ref to features> ] )
 
-Performs the shaping.
+Performs the actual shape operation.
 
 I<features> is a reference to an array of feature strings. The
 features will be I<added> to the list of features already set with
@@ -260,6 +420,15 @@ sub shaper {
     }
 
     hb_buffer_clear_contents( $self->{buffer} );
+
+    for ( qw( language script direction ) ) {
+	next unless $self->{$_};
+	# All setters return undef if something wrong.
+	no strict 'refs';
+	my $action = "hb_buffer_set_$_";
+	$action->( $self->{buffer}, $self->{$_} )
+	  || croak("Invalid $_: \"$self->{$_}\"" );
+    }
     hb_buffer_add_utf8( $self->{buffer}, $self->{text} );
 
     hb_buffer_guess_segment_properties( $self->{buffer} );
